/**
 * @file   mbc.h
 * @brief  for Multi-Bytes-Charactor Code including   '\'. (SJIS, BIG5, GBK)
 * @author Masashi Kitamura
 * @license Boost Software License 1.0
 * @note
 * - It is mainly for Windows.
 */
#ifndef  MBC_H_INCLUDED
#define  MBC_H_INCLUDED

#ifdef HAVE_MBSTRING_H

#include <mbstring.h>

#else /* ! HAVE_MBSTRING_H */

#ifdef _WIN32

#include <windows.h>
#define  _ismbblead(c)	   IsDBCSLeadByte((unsigned char)(c))

#else /* linux(unix)... ex. LANG=ja_JP.SJIS */

#ifdef HAVE_STRINGS_H
# include <strings.h>
#else
# include <string.h>
#endif
#include <stdlib.h>

#define  _ismbblead(c)	   Mbc_SjisBig5Gbk<0>::ismbblead(c)

template<unsigned DMY>
class Mbc_SjisBig5Gbk
{
public:
   static int  ismbblead(unsigned char c)
   {
	  return c < 0x80 ? 0 : ismbblead_sjisBig5Gbk(c);
   }

private:
   static int			ismbblead_sjisBig5Gbk(unsigned c);
   static void			init();
   static unsigned char type_;
};


template<unsigned DMY>
unsigned char Mbc_SjisBig5Gbk<DMY>::type_ =  0;


template<unsigned DMY>
int   Mbc_SjisBig5Gbk<DMY>::ismbblead_sjisBig5Gbk(unsigned c)
{
  RETRY:
   switch (type_)
   {
   case 0 /* INIT */:
	  init();
	  goto	RETRY;
   case 2 /* SJIS */:
	  return ((c >= 0x81 && c <= 0x9F)	 || (c >= 0xE0 && c <= 0xFC));
   case 3 /* BIG5 */:
	  return ((c >= 0xA1 && c <= 0xC6)	 || (c >= 0xC9 && c <= 0xF9));
   case 4 /* GBK  */:
	  return (c >=	0x81 &&  c <= 0xFE);
   default:
	  return 0;
   }
}


template<unsigned DMY>
void Mbc_SjisBig5Gbk<DMY>::init()
{
   const char*	  lang = getenv("LANG");
   type_ =	1; /* ascii etc...	 */
   if (lang)
   {
	  const char* p    =   strrchr(lang, '.');
	  if (p)
	  {
		 ++p;
		 if (strncasecmp(p, "sjis", 4) == 0)
		 {
			type_ = 2;
		 }
		 else if (strncasecmp(p, "big5", 4) ==	 0)
		 {
			type_ = 3;
		 }
		 else if (!strncasecmp(p, "gbk", 3) || !strncasecmp(p, "gb18030", 7))
		 {
			type_ = 4;
		 }
	  }
   }
}

#endif	 /* _WIN32 */


/**   Charactor type in line[pos]. 0=ascii 1=DBC-hi 2=DBC-lo -1:bad
 * ( subset )
 */
static inline int _mbsbtype(const unsigned char* line, unsigned pos)
{
   unsigned i;
   for (i = 0;	i <   pos; i++)
   {
	  unsigned c = line[i];
	  if (c == 0)
	  {
		 return -1;
	  }
	  else if (_ismbblead(line[i]) && line[i+1])
	  {
		 if (i == pos - 1)
		 {
			// if (_ismbbtrail(line[i+1]) == 0) return -1;
			return 2;
		 }
		 i++;
	  }
   }
   return (line[pos]) ? (_ismbblead(line[pos]) != 0) :	-1;
}

#endif /* HAVE_MBSTRING_H */


#endif /* MBC_H_INCLUDED */
